<html>
<head>
	<title>PHP Object Oriented Programming</title>
</head>

<body>

<h2>PHP Object Oriented Programming</h2>

<h3>Introduction to Object Oriented Thinking</h3>

<p>This tutorial is designed to give those familiar with PHP programming an introduction to Object Oriented Programming.  It is not intended as a beginner tutorial, nor is it intended to be an all-encompassing tutorial on the inner-workings of OOP.  However, if you're unfamiliar with the ideas behind and the implementation of classes and objects, this may be a good place for you to start.</p>

<p>This tutorial will use as an example a user registration class (more on classes shortly).  This class is a real world example of the type of implementation a PHP programmer may choose to use when programming a web-based community or forum system, but could also be expanded to a customer registration database or something similar.  While we choose to use this as an example, remember, the concepts here can be used on virtually any programming project if your chosen language supports Object Oriented Programming.</p>

<p>In order to complete this tutorial, you will need a working installation of PHP with MySQL support installed and access to a MySQL database.  The database will be used to store the user data.  You can download a zip file of all the examples used in this tutorial here:  <a href="phpClassTutorial.zip">phpClassTutorial.zip</a> (less than 4k).</p>

<p>So, what <i>is</i> Object Oriented Programming (or OOP as it's generally referred to)?  OOP is a different way of <i>thinking</i> about programming.  Once you understand the concepts behind it, the implementation is the easy part.  In OOP, the idea is to abstract what you're trying to accomplish to more resemble real-world things, or objects.</p>

<p>Many introductions to these concepts use a car stereo as an example.  From a real world perspective, it's a very valid example, so it is discussed here briefly.  A car stereo is a "thing" or "object" that you as a user generally know how to <i>use</i>, but probably don't understand all the technical mumbo-jumbo that happens behind-the-scenes.  For example, when you switch to a certain radio, you most likely don't know <i>how</i> the electronic components receive different frequencies of radio waves and how those are turned into the music that you hear.</p>

<p>This is one of the main "features" of OOP.  A person can <i>use</i> one of your classes without actually having to understand how it works internally.  All the user has to know is how to work the controls.  For example, the volume control, the station selection knob, etc.</p>

<p>This describes a "feature," but doesn't really explain why you would want something like this.  The idea is that there are specific data elements that your class or object will contain (for example, the frequency and the volume), and there are also specific ways those data elements can be manipulated.  Instead of keeping these related data members and functions separate, OOP dictates that since they are so related, they should be packaged together.  For example, in our previous example, our data elements would consist of radio frequency and volume level, and the functions we would write to manipulate those data members would be volumeUp(), volumeDown(), frequencyUp(), and frequencyDown().</p>

<p>This is all well and good, but how does this help with PHP programming?  Frankly, the car stereo example didn't help me much, either, at first.  It's important to work with an example that deals with the specific tasks PHP was designed to deal with.  So without further ado, let's dive into our user registration class.</p>

<h3>Class and Object Specifics</h3>

<p>The word "class" has been thrown around a few times, but hasn't been clearly defined yet.  This is a very difficult thing to explain with just words, but once the design is hammered out, it's relatively easy to understand.  So, if you don't understand the next few sentences, don't worry about it, classes and objects will become clear as this tutorial progresses.  Think of a "class" as sort of a template for data.  If we have a user registration class, say our template will contain 3 variables: $userID, $userName, and $userPassword.  The "class" then, is just a way to define a structured placeholder for these variables - when you create a class, no variables are assigned, and no space is set aside in memory.  A "class" is simply a template.  Furthering that concept, a class also contains functions (very similar to standard PHP functions) that are specifically designed to manipulate the variables that are part of the class.  So the class that contains $userID, $userName, and $userPassword would probably also contain functions such as verifyPassword(), changeUserName(), changePassword(), etc.  Some classes are absolutely massive and scope, and some are very small, but most contain 10-20 data members and about 3 times that many functions.</p>

<p>Some notes on OOP and terminology:  Functions defined inside your class are generally called "methods."  Don't ask me why, it's probably just more techno-babble to make the guys that thought it up feel smarter.  Variables inside the classes (or 'objects' - more on this shortly) are sometimes referred to as data members.  This tutorial will use these terms interchangeably, so please don't get confused!</p>

<p>As mentioned above, a class is just a template for these data members and methods.  So, what good is a template, and how do you use it?  The idea is that you can create any number of "objects" based on a single "class."  So at the top of your document, you'd have your class definition, and then below, you could instantiate any number of "objects" based on that class.  So if your class is called "User" you may create an object based on that class called $myUser or $myOtherUser or both in one script.  You can even create an array of fifty or a hundred objects, if that's what you needed to do.</p>

<p>So to concrete this concept, say we had our "User" class defined with the methods mentioned above.  When someone accesses a page on our site, they are required to enter their user name and their password.  When they hit the "Submit" button, our User class gets to do some work!  The first thing we need to do is instantiate an object based on the User class.  This is done with the following statement.</p>

<code>$myUser = new User;</code>

<p>Very simple.  Now, we need to assign the values the user entered to the data members of the $myUser object.  This is done with very straight-forward syntax.  C/C++ programmers will be familiar with the pointer-to-member operator.</p>

<code>$myUser->userName = $inputtedUserName;<br>
$myUser->userPassword = $inputtedUserPassword;</code>

<p>This isn't the way data is typically assigned to the data members of the object, but it will do for this example.  More on this later.  Now that the data is there, we want our script to verify that they entered a correct user name and password combination.  This is done simply by calling the verifyPassword() function/method mentioned earlier.  Remember, all we need to know at this point is how to use the class - we don't need to know what is actually taking place behind the scenes.  In this example, the only data the object needs to verify the password is the user name and password.  Then, we simply call the correct method.</p>

<code>$myUser->verifyPassword();</code>

<p>Depending on how the class is designed, the function could return a value (true if the password was good, false if not), directly error out and kill the script (not recommended - more on this later), or take any other action determined by the person who programmed the class.</p>

<p>Now that a basic introduction to OOP, classes, data members, and methods has been completed, we can begin the design phase of our user registration class.  If the above sections didn't make much sense, please continue reading - a lot of the time, real code and working examples are the best way to learn a new concept.</p>

<h3>Class Design</h3>

<p>When designing a class, the first step is generally brainstorming a basic idea of what the class is supposed to do.  In our example, the user registration class should handle everything from registering and deleting users to manipulating all the data members that are part of the class.  As mentioned above, we also want to include a method for verifying that a user has access to a specific page.</p>

<p>Here are the data members this example will use.  Remember, you can have a virtually unlimited number of data elements, so this example can be easily expanded upon.  Once we figure out what data we're going to store, we can add a table to our database to hold it.</p>

<ul>
	<li><code>$userID</code> - assigned automatically by the auto_increment field in the MySQL database.
	<li><code>$userName</code> - limited to 32 characters for this example.
	<li><code>$userPassword</code> - also limited to 32 characters.
	<li><code>$dbHost</code> - MySQL server host.
	<li><code>$dbUser</code> - MySQL user.
	<li><code>$dbName</code> - MySQL database name.
	<li><code>$dbPass</code> - MySQL database password.
	<li><code>$dbUserTable</code> - the table that will hold our data.
</ul>

<p>That's enough data for this tutorial.  Now we need to figure out what methods our class will need to manipulate the above data.  Below is not an exhaustive list, we may need more that will be discovered as we go along.  Included with each method is the argument list it should be designed to take.</p>

<ul>
	<li><code>registerUser($userName, $userPassword)</code> - instead of assigning these values directly, the registerUser() method will be designed to make sure they are the correct length, don't use any special characters, aren't already taken, etc.
	<li><code>verifyPassword()</code> - assume we've already assigned the values to the variables.
	<li><code>changePassword($newPassword)</code> - assume old password is already registered to the userPassword variable.
	<li><code>displayUserInfo()</code> - a quick and dirty method to allow us to view the variables in our object.
</ul>

<p>Those are enough for this example.  Remember, you can add as many methods as you want.</p>

<p>So with the above data members, here is the table schema I used to store the data.</p>

<code><pre>create table usersExample (
	userID int not null primary key auto_increment,
	userName varchar(32) not null,
	userPassword varchar(32) not null);</pre></code>

<p>Once you have your table set up, it's time to start coding!</p>

<h3>Coding a 'Skeleton' of the User Class</h3>

<p>I find it helpful to code a basic 'Skeleton' of my classes as a roadmap.  It helps keeps my thoughts and code organized.</p>

<p>The first thing we need to do is create a UserClass.php file which will contain our class definition.  This file will then be included (using the include() function) in each of our pages that need it.  Please read through the following code very carefully, paying special attention to the comments.  Consider the comments part of this tutorial - if you skip them, you will be lost!</p>

<?
show_source('UserSkeleton.php');
?>

<h3>Details on Class/Object Variables</h3>

<p>When you create a class, it's a template.  In order to use the template, you have to instantiate an object based on that template.  Once your object is created, the variables inside that object are accessible to you anywhere in your script.  This is a deviation from a full implementation of OOP because generally, these variables are supposed to be hidden or 'private' - meaning, the only functions that can access the variables are those inside the class definition.</p>

<p>So, when you instantiate an object using:</p>

<code>$myUser = new User;</code>

<p>$myUser now contains all the variables listed under the 'var' section of your class definition.  You can access these variables using the following syntax.  This was already covered above, but we're going to dig a little deeper.</p>

<code>echo $myUser->userName;</code>

<p>Concerning scope... It's perfectly okay to have a variable called <code>$userName</code> in your main script - it will not conflict with the one in your $myUser object.  Also, if you create another instance of the User class called <code>$myOtherUser</code>, each instance will have it's <i>own</i> userName data member.  So <code>$userName</code>, <code>$myUser->userName</code>, and <code>$myOtherUser->userName</code> are all different variables.</p>

<p><b>Important:</b> There is special syntax required inside methods/class functions to access variables inside a specified class.  If that didn't make sense, I'll try different wording.  If you have a User class that contains the variables we listed above, many of your methods will need to directly access those variables (in order to manipulate, update, etc.).  Inside those functions, there is special syntax required to let the functions know that they are accessing the variables belonging to the specific <i>object</i> that's calling them, rather than a global variable or a data member from another instance of the object.</p>

<p>That still seems very complicated, when in fact, it's not.  Let's keep trying, to make sure this is crystal clear.  Each function you write only exists once in memory.  When an object calls that function, the function needs to know which data it's supposed to be accessing or manipulating.  So, if you have a $myUser->userName variable and a $myOtherUser->userName variable, how does the verifyPassword() function know which instance of the User class it's supposed to be manipulating?</p>

<p>Since you call the function with a specific instance of the object, for example, <code>$myUser->verifyPassword()</code> or <code>$myOtherUser->verifyPassword()</code> you would assume that the function would know which object actually called it.  To a certain extent, this is true.  However, since when you are actually coding the class, there are no objects actually instantiated, you need to have notation to tell the function "use the data from the object that is calling you."  This is accomplished with the "this pointer."  The term "this pointer" is taken from C++, a popular compiled language.<p>

<p>The "this pointer" is an invisible variable passed to the method (function) by the calling object.  It is simply a variable that points back to the calling object to let the function know which instance of the class actually called it.  This may seem overly complex, but in practice, it's actually very simple.  Let's look at some sample code.</p>

<p>Below is the displayUserInfo() method.  The following code should be placed in the correct spot in your skeleton.  Basically, right now we are making the displayUserInfo() method actually <i>do</i> something.  Remember, the beginning and end portions of our User class definition will appear above and below this.  Just as before, be sure to read the comments, they are very important.</p>

<?
show_source('displayUserInfo.php');
?>

<p>Now that we know the special notation required inside our methods, we can finish writing the remaining methods.</p>

<h3>Understanding and Using Constructors</h3>

<p>A "constructor" is simply a method inside a class that gets run automatically each time an instance of the class is created.  For instance, when a script says:</p>

<code>$myUser = new User;</code>

<p>If the User class had a constructor, it would get executed at that point.  Constructors are very important and have many uses.  In our example, we will need a constructor to set all the database values.  These values will be the same for each object, so it makes sense to set them in a constructor.</p>

<p>A function is designated a constructor by virtue of sharing its name with the name of the class.  So, since our class is called 'User' our constructor will be called 'User' as well.  The 'User' function should set all the variables needed for the database connection.</p>

<p>Please pay special attention to the constructor function declaration in the code displayed in the next section.</p>

<h3>User Class Methods</h3>

<p>This section contains the source for the remaining methods of our class.  It's important that you have an understanding of how they work and why they work the way they do.  The best way to gain this understanding is to read through the source and the comments especially.  The code here has been kept very small because it's an academic example.  In a real-world scenario, these methods would be required to do a lot more error checking, etc.</p>

<?
show_source('UserClass.php');
?>

<p>That looks like it for our User class.  Does that make you a proficient OOP programmer?  Not yet.  Now, we'll explore some examples of how to actually use this class to make our programming easier.</p>

<h3>Examples Using the User Class</h3>

<p>The first example we'll use is how to register a user into the database.  This is a fairly simple process, and will require two files.  The first file, <code>registerForm.html</code>, is simply an HTML file containing a form.  The second file, <code>processRegister.php</code>, actually contains the logic to register the user.</p>

<?
show_source('registerForm.html');
?>

<p>Now for the <code>processRegister.php</code> file which processes the data from the previous form.</p>

<?
show_source('processRegister.php');
?>

<p>The next example is a bit more complex (but still not overly so).  This time, we will write a set of scripts to allow the user to change his/her password.  This requires them to type in their user name, their old password, and a new password.  The new password will only be changed if the old password is correct!  This example will also contain two files, <code>passwordForm.html</code> and <code>processPassword.php</code>.</p>

<?
show_source('passwordForm.html');
?>

<p>Now for the <code>processpassword.php</code> file which processes the data from the previous form.</p>

<?
show_source('processPassword.php');
?>

<p>Hopefully by now you can see that it's very easy to use the classes once they're programmed.  We don't have to know how the class operates internally, all we have to know is what data members it has and what methods there are.  Notice that in our *.php files, there isn't any complex code such as database connections, queries, etc.  Since it's all defined in the class, we don't have to worry about typing database connection code each time we need it - the class connects to the database when it needs to.</p>

<p>Please understand that this is an academic example, one not suited to regular use on a public web site.  We haven't done any error checking on the data, the table isn't properly indexed, and we're obviously not storing enough data for a real world application.  However, this can be used as a base to build upon.</p>

<h3>Encapsulation, Polymorphism, and Inheritance</h3>

<p>Encapsulation, polymorphism, and inheritance are three buzz-words that simply must be addressed when dealing with Object Oriented Programming.  However, PHP doesn't exactly contain a 'full' implementation of OOP, so these words don't exactly apply like they do to other languages.  Here is a quick run-down; not a lot of detail, but enough to give you some basic information.</p>

<ul>
	<li><b>Encapsulation</b> - This is simply a term for having your data members and methods together in one nifty form called a class.
	<li><b>Polymorphism</b> - This is a term used in OOP where a function can act differently based on its context.  For example, if the verifyPassword() function was passed 2 values, it would act differently than if it was only passed one.
	<li><b>Inheritance</b> - This is a great function of OOP that allows you to create 'child' or 'sub' classes based off an already existing class.  For example, we could create a child class for our User class that added more methods and data members, but didn't interfere with the ones we already had.
</ul>

<p>All of these terms are important and may be covered PHP-style in later tutorials.</p>

<hr noshade>

<p><font size="-1">Author: Brian Lozier (<a href="mailto:brian@massassi.net">brian@massassi.net</a>)<br>This document &copy; Copyright 2001 Brian Lozier - All rights reserved.</font></p>

</body>
</html>